// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Project
#include <anna/core/core.hpp>
#include <anna/diameter/functions.hpp>
#include <anna/time/functions.hpp>
#include <anna/diameter/codec/Engine.hpp>
#include <anna/diameter.comm/Response.hpp>
#include <anna/diameter.comm/ClientSession.hpp>
#include <anna/diameter.comm/Server.hpp>
#include <anna/diameter.comm/OriginHost.hpp>

// Process
#include <MyLocalServer.hpp>
#include <MyDiameterEngine.hpp>
#include <MyDiameterEntity.hpp>
#include <rxSimpleTest.hpp>


void MyLocalServer::eventRequest(anna::diameter::comm::ServerSession *serverSession, const anna::DataBlock &message)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("rxSimpleTest::MyLocalServer", "eventRequest", ANNA_FILE_LOCATION));
  // Performance stats:
  RxSimpleTest& my_app = static_cast <RxSimpleTest&>(anna::app::functions::getApp());
  anna::diameter::comm::OriginHost *my_node = my_app.getOriginHost(getEngine()->getOriginHostName());
  anna::diameter::codec::Engine *codecEngine = my_node->getCodecEngine();

  // CommandId:
  anna::diameter::CommandId cid = anna::diameter::codec::functions::getCommandId(message);
  LOGDEBUG
  (
    std::string msg = "Request received: ";
    msg += anna::diameter::functions::commandIdAsPairString(cid);
    msg += " | DiameterServer: ";
    msg += anna::functions::socketLiteralAsString(serverSession->getAddress(), serverSession->getPort());
    msg += " | EventTime: ";
    msg += anna::time::functions::currentTimeAsString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  anna::diameter::codec::Message codecMsg;
  anna::diameter::codec::Message codecAnsMsg;
  anna::diameter::codec::Message *answer_message = (anna::diameter::codec::Message *)&codecAnsMsg;
  my_app.getCommunicator()->prepareAnswer(answer_message, message);

  anna::diameter::comm::Message *msg;
  try {
    msg = my_node->createCommMessage();
    msg->setBody(answer_message->code());
    /* response = NULL =*/serverSession->send(msg);
  } catch(anna::RuntimeException &ex) {
    ex.trace();
  }

  // release msg
  my_node->releaseCommMessage(msg);
}

void MyLocalServer::eventResponse(const anna::diameter::comm::Response &response)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("rxSimpleTest::MyLocalServer", "eventResponse", ANNA_FILE_LOCATION));
  RxSimpleTest& my_app = static_cast <RxSimpleTest&>(anna::app::functions::getApp());
  anna::diameter::comm::OriginHost * my_node = my_app.getOriginHost(getEngine()->getOriginHostName());
  anna::diameter::comm::ClassCode::_v code = response.getClassCode();
  anna::diameter::comm::Response::ResultCode::_v result = response.getResultCode();
  anna::diameter::comm::Message* request = const_cast<anna::diameter::comm::Message*>(response.getRequest());
  const anna::DataBlock* message = response.getMessage();
  const anna::diameter::comm::ServerSession *serverSession = static_cast<const anna::diameter::comm::ServerSession *>(response.getSession());
  bool contextExpired = (result == anna::diameter::comm::Response::ResultCode::Timeout);
  bool isUnavailable = (result == anna::diameter::comm::Response::ResultCode::DiameterUnavailable);
  bool isOK = (result == anna::diameter::comm::Response::ResultCode::Success);

  // CommandId:
  anna::diameter::CommandId request_cid = request->getCommandId();
  LOGDEBUG
  (
    std::string msg = "Response received for original diameter request: ";
    msg += anna::diameter::functions::commandIdAsPairString(request_cid);
    msg += " | Response: ";
    msg += response.asString();
    msg += " | LocalServer: ";
    msg += anna::functions::socketLiteralAsString(serverSession->getAddress(), serverSession->getPort());
    msg += " | EventTime: ";
    msg += anna::time::functions::currentTimeAsString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  if(isUnavailable) {
    //if (isApplicationMessage)
    LOGWARNING(anna::Logger::warning("Diameter client unavailable for Diameter Request", ANNA_FILE_LOCATION));
  }

  if(contextExpired) {
    //if (isApplicationMessage)
    LOGWARNING(anna::Logger::warning("Context Expired for Diameter Request which was sent to the client", ANNA_FILE_LOCATION));
  }

  if(isOK) {
    LOGDEBUG(
      std::string msg = "Received response for diameter message:  ";
      msg += anna::diameter::functions::commandIdAsPairString(request_cid);
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
  }
}

void MyLocalServer::eventUnknownResponse(anna::diameter::comm::ServerSession *serverSession, const anna::DataBlock &message)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("rxSimpleTest::MyLocalServer", "eventUnknownResponse", ANNA_FILE_LOCATION));
  // Performance stats:
  RxSimpleTest& my_app = static_cast <RxSimpleTest&>(anna::app::functions::getApp());
  anna::diameter::comm::OriginHost *my_node = my_app.getOriginHost(getEngine()->getOriginHostName());
  // CommandId:
  anna::diameter::CommandId cid = anna::diameter::codec::functions::getCommandId(message);
  LOGDEBUG
  (
    std::string msg = "Out-of-context response received from client: ";
    msg += anna::diameter::functions::commandIdAsPairString(cid);
    msg += " | DiameterServer: ";
    msg += anna::functions::socketLiteralAsString(serverSession->getAddress(), serverSession->getPort());
    msg += " | EventTime: ";
    msg += anna::time::functions::currentTimeAsString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

void MyLocalServer::eventDPA(anna::diameter::comm::ServerSession *serverSession, const anna::DataBlock &message)
throw(anna::RuntimeException) {
  LOGMETHOD(anna::TraceMethod tm("rxSimpleTest::MyLocalServer", "eventDPA", ANNA_FILE_LOCATION));
  // Performance stats:
  RxSimpleTest& my_app = static_cast <RxSimpleTest&>(anna::app::functions::getApp());
  anna::diameter::comm::OriginHost *my_node = my_app.getOriginHost(getEngine()->getOriginHostName());
  // CommandId:
  anna::diameter::CommandId cid = anna::diameter::codec::functions::getCommandId(message);
  LOGDEBUG
  (
    std::string msg = "Disconnect-Peer-Answer response received from client: ";
    msg += anna::diameter::functions::commandIdAsPairString(cid);
    msg += " | DiameterServer: ";
    msg += anna::functions::socketLiteralAsString(serverSession->getAddress(), serverSession->getPort());
    msg += " | EventTime: ";
    msg += anna::time::functions::currentTimeAsString();
    anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}
