// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_internal_LocalConnection_hpp
#define anna_comm_internal_LocalConnection_hpp

namespace anna {

namespace xml {
class Node;
}

namespace comm {

class ServerSocket;
class ClientSocket;

namespace handler {
class ServerSocket;
}

/**
   Implementa los socket locales, es decir, conexiones aceptadas por los ServerSocket que
   estan activos en nuestra aplicacion.
*/
class LocalConnection {
public:
  /**
     Constructor.
  */
  LocalConnection() : a_serverSocket(NULL), a_clientSocket(NULL) {;}

  /**
     Devuelve el servidor de socket a partir del cual se cre este socket cliente.Puede ser NULL si este
     socket cliente no est�asociado a ningn servidor de sockets.
     \return El servidor de socket a partir del cual se cre este socket cliente.
  */
  ServerSocket* getServerSocket() const throw() { return a_serverSocket; }

  /**
     Devuelve el socket cliente  asociado a esta conexion. Puede ser NULL si la conexion no ha sido establecida.
     \return El socket cliente asociado a esta conexion.
  */
  ClientSocket* getClientSocket() throw() { return a_clientSocket; }

  /**
     Devuelve una cadena con la informacin referente a este socket.
     \return Una cadena con la informacin referente a este socket.
  */
  std::string asString() const throw();

  /**
     Devuelve un nodo XML con la informacin referente a este objeto.
     \param parent Nodo XML a partir del cual introducir la informacin.
     \return Un nodo XML con la informacin referente a este objeto.
  */
  xml::Node* asXML(xml::Node* parent) const throw(RuntimeException);

  /**
     Devuelve el nombre logico de esta clase.
     \return el nombre logico de esta clase.
  */
  static const char* className() throw() { return "anna::comm::LocalConnection"; }

private:
  ServerSocket* a_serverSocket;
  ClientSocket* a_clientSocket;

  void setServerSocket(ServerSocket* serverSocket) throw() { a_serverSocket = serverSocket; }
  void setClientSocket(ClientSocket* clientSocket) throw() { a_clientSocket = clientSocket; }

  friend class ServerSocket;
  friend class handler::ServerSocket;
};

}
}

#endif

