// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_tracing_TraceFunction_hpp
#define anna_core_tracing_TraceFunction_hpp

#include <anna/core/tracing/Logger.hpp>

namespace anna {

/**
  Trazas de funciones.

  Graba una traza en el historico de operaciones por cada entrada y salida de una determinada
  funcion.
*/
class TraceFunction {
public:
  /**
    Constructor.
    @param functionName Nombre de la funcion desde la que se invoca a este constructor.
    @param fromFile Nombre del fichero desde el que se invoca a este constructor. Normalmente
    sera el indicado por la macro de compilacion __FILE__.
    @param fromLine Numero de linea del fichero desde la que se invoca a este constructor.
    Normalmente sera el indicado por la macro de compilacion __LINE__.
  */
  TraceFunction(const char* functionName, const char* fromFile, const int fromLine) :
    a_functionName(functionName),
    a_fromFile(fromFile),
    a_ok(false) {
    if(Logger::isActive(Logger::Debug) == true) {
      a_ok = true;
      Logger::write(Logger::Debug, functionName, "begin", fromFile, fromLine);
    }
  }

  /**
     Destructor.
  */
  ~TraceFunction() {
    if(a_ok == true && Logger::isActive(Logger::Debug) == true)
      Logger::write(Logger::Debug, a_functionName, "end", a_fromFile, 0);
  }

private:
  const char* a_functionName;
  const char* a_fromFile;
  bool a_ok;
};

} //namespace anna

#endif

