// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_tracing_TraceWriterObserver_hpp
#define anna_core_tracing_TraceWriterObserver_hpp

#include <string>

namespace anna {

class TraceWriter;

/**
  Interfaz que permite conocer cuando el TraceWriter asociado a nuestro sistema de trazas
  abre o cierra el fichero que usa para la salida de datos.

  Una vez asociado al TraceWriter esta instancia debera estar disponible durante toda
  la ejecucion.

   \see TraceWriter::attach
*/
class TraceWriterObserver {
public:
  /**
     Devuelve el nombre logico de este observador.
  */
  const char* getName() const throw() { return a_name; }

  /**
     Devuelve una cadena con la informacion relevante de esta clase.
     \return una cadena con la informacion relevante de esta clase.
  */
  std::string asString() const throw() {
    std::string result("anna::TraceWriterObserver { Name: ");
    result += a_name;
    return result += " }";
  }

protected:
  /**
     Constructor.
     \param name Nombre logico de este observador.
  */
  TraceWriterObserver(const char* name) : a_name(name) {;}

  /**
     Metodo que hay que reescribir para tratar la notificacion de apertura del fichero de trazas.
     \param fd \em File \em Descriptor usado por TraceWriter para escribir en el fichero de salida.
     \warning El fichero usado para escribir las trazas puede abrirse un numero indeterminado
     de veces durante la ejecucion de nuestra aplicacion.
  */
  virtual void handleOpen(const int fd) throw() = 0;

  /**
     Metodo que hay que reescribir para tratar la notificacion de cierre del fichero de trazas.
     \param fd \em File \em Descriptor usado hasta el momento por TraceWriter para escribir en el
     fichero de salida.
     \warning El fichero usado para escribir las trazas puede cerrar un numero indeterminado
     de veces durante la ejecucion de nuestra aplicacion.
  */
  virtual void handleClose(const int fd) throw() = 0;

private:
  const char* a_name;

  friend class TraceWriter;
};

}

#endif


