// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Environment_hpp
#define anna_core_util_Environment_hpp

#include <string>
#include <map>

#include <anna/core/RuntimeException.hpp>
#include <anna/core/Singleton.hpp>


namespace anna {

/**
  Environment variables reader
*/
class Environment : public Singleton <Environment> {
public:

  /**
  * Destructor.
  */
  ~Environment() {;}

  /**
    Parses the environment data (all variables available) when process was started.
    @param envp Environment array passed on main function as third argument.
    Cache data is cleared if NULL passed, allowing to get current environment values for variables.
  */
  void initialize(char **envp = NULL) throw();

  /**
    Return associated value (could be empty). This value could be cached at initialization (envp array from main function), if not, would be
    cached here.

    @param variableName Environment variable name.
    @param exceptionIfMissing When enabled, an exception is launched for missing variables. Empty string in other case (default behaviour).

    @return Environment value.
  */
  std::string getValue(const char* variableName, bool exceptionIfMissing = false) throw(RuntimeException);
  std::string getValue(const std::string &variableName, bool exceptionIfMissing = false) throw(RuntimeException);

  /**
    Sets an environment variable. If an empty variable name is provided, or environment set operation fails,
    an exception will be launched.

    @param name Variable name.
    @param value Variable value
    @param overwrite Overwrite an existing variable name/value or keep old value if exists
  */
  void setVariable(const std::string &name, const std::string &value, bool overwrite = true) throw(RuntimeException);

  /**
    Unsets an environment variable (different than set empty string). If an empty variable name is provided,
    or environment set operation fails, an exception will be launched.

    @param name Variable name. If empty, nothing is done.
  */
  void unsetVariable(const std::string &name) throw(RuntimeException);


private:

  std::map < std::string /* variable name */, std::string /* variable value */ > a_vars;

  Environment() {;}

  friend class Singleton <Environment>;
};

}

#endif

