// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_ldap_Attribute_hpp
#define anna_ldap_Attribute_hpp

#include <string>

#include <anna/core/util/Recycler.hpp>

namespace anna {

namespace ldap {

class Session;
class Response;

/**
   Modela los atributos retornados en una peticion de busqueda.

   Recordar que en LDAP un mismo atributo puede tener asociados un numero indeterminado de valores.

   \see ldap::Response
*/
class Attribute {
public:
  typedef Recycler<std::string> value_container;
  typedef value_container::const_iterator const_value_iterator;

  /**
     Devuelve el nombre del atributo.
     \return El nombre del atributo.
  */
  const std::string& getName() const throw() { return a_name; }

  /**
     Devuelve el iterador al comienzo de la lista de valores asociados a este atributo.
     \return El iterador al comienzo de la lista de valores asociados a este atributo.
  */
  const_value_iterator value_begin() const throw() { return a_values.begin(); }

  /**
     Devuelve el iterador al final de la lista de valores asociados a este atributo.
     \return El iterador al final de la lista de valores asociados a este atributo.
  */
  const_value_iterator value_end() const throw() { return a_values.end(); }

  /**
     Devuelve el valor asociado al iterador recibido como parametro.
     \param ii Instancia del iterador usado para recorrer los datos. Estara comprendido
     entre [#value_begin, #value_end).
     \return El valor referenciado por el iterador.
  */
  static const std::string& value(const_value_iterator ii) throw() { return *value_container::data(ii); }

private:
  std::string a_name;
  value_container a_values;

  Attribute() {;}

  void setName(const std::string& name) throw() { a_name = name; }

  void add(const std::string& value) throw(RuntimeException) {
    std::string* newValue = a_values.create();
    *newValue = value;
  }
  void clear() throw() { a_name.clear(); a_values.clear(); }

  friend class Session;
  friend class Response;
  friend class Allocator<Attribute>;
};

}
}

#endif

