// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_time_functions_hpp
#define anna_time_functions_hpp

#include <string>


namespace anna {

namespace time {

class Timezone;

class functions {
public:

  // sets

  /**
     Initializes global data used on time module. It must be firstly invoked on this module
  */
  static void initialize() throw();

  /**
     Initializes reference timestamp used for statistics purposes and neccessary on limited 32-bits architectures
     in order to avoid overflow for unix timestamps. Time module initizalization timestamp remains if this is not called.

     @param secondsTimestamp Reference timestamp in seconds. By default current is set.
  */
  static void setControlPoint(unsigned long long int secondsTimestamp = 0) throw();

  /**
     Gets the seconds reference established by mean #setControlPoint
  */
  static unsigned long long int getSecondsReference() throw();



  // gets

  /**
     Gets initialization state
  */
  static bool initialized(void) throw();

  /**
     Gets the host user-supplied timezone configuration
  */
  static const Timezone & getSystemTimezone(void) throw();


  /**
     Get unix timestamp in seconds
  */
  static unsigned long long int unixSeconds() throw();


  /**
     Get unix timestamp in milliseconds
  */
  static unsigned long long int unixMilliseconds() throw();

  /**
     Get the lapsed milliseconds from the last control point (#setControlPoint) or time module initialization.
     Used for statistics purposes (performance analysis), specially on 32-bits limited architectures in order
     to avoid overflow on timestamp structures.
  */
  static unsigned long long int lapsedMilliseconds() throw();

  /**
     Get unix timestamp in microseconds
  */
  static unsigned long long int unixMicroseconds() throw();


  /**
     Get string time representation in format 'HH:mm:ss \<usecs\>'
     Useful for trace accurated time for events.
  */
  static std::string currentTimeAsString(void) throw();
};

}
}

#endif

