// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <poll.h>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/mt/Semaphore.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/Handler.hpp>
#include <anna/comm/Transport.hpp>

using namespace std;
using namespace anna;

//------------------------------------------------------------------------------------------------
// En modo MT aprovechamos las capacidades que tiene esta clase por el hecho de heredar de
// anna::Runnable -> invocaremos a do_action atraves de Runnnable::run.
//
// En modo ST invocamos directamente al Handler::apply.
//------------------------------------------------------------------------------------------------
void comm::Handler::do_action()
throw(RuntimeException) {
  pollfd pollfd;
  pollfd.fd = getfd();
  pollfd.events = POLLIN | POLLRDNORM;
  const int npoll = poll(&pollfd, 1, 1000);

  if(npoll == -1 && errno != EINTR) {
    const int xerrno(errno);
    string msg(asString());
    msg += " | Error checking messages";
    throw RuntimeException(msg, xerrno, ANNA_FILE_LOCATION);
  }

  bool done(false);

  if(supportTimeout() == false) {
    if(npoll == 1 && (pollfd.revents & (POLLIN | POLLRDNORM)) != 0) {
      done = true;
      a_loop = 0;

      try {
        apply();
      } catch(RuntimeException& ex) {
        ex.trace();
      }
    }
    /*
     * En la prctica este cdigo slo se ejecutar en modo MT, ya que en modo ST, slo se invocar a Handler::do_action
     * cuando se compruebe que actividad en el 'fd' asociado a este manejador.
     *
     * Sin embargo en MT el thread est ejecutando este cdigo peridicamente.
     */
    else if(++ a_loop == 5) {
      a_loop = 0;

      try {
        testClose();
      } catch(RuntimeException& ex) {
        ex.trace();
      }
    }
  } else {
    Microsecond now(anna::functions::hardwareClock());

    if(npoll == 1 && (pollfd.revents & (POLLIN | POLLRDNORM)) != 0) {
      done = true;

      try {
        beat(now);
        apply();
      } catch(RuntimeException& ex) {
        ex.trace();
      }
    } else if(isTimeout(now) == true) {
      done = true;
      LOGWARNING(
        string msg(asString());
        msg += " | Closed due to inactivity";
        Logger::warning(msg, ANNA_FILE_LOCATION);
      );
      a_communicator->detach(this);
    }
  }

  /**
   * Parece ser que puede ser que adems de los POLLIN y POLLRDNORN que estamos comprobando
   * pueden aparecer ms bits, POLLINVAL (p.e) lo que provocara
   */
  if(npoll == 1 && done == false) {
    LOGWARNING(
      string msg(asString());
      msg += " | Detected unsupported event";
      Logger::warning(msg, ANNA_FILE_LOCATION);
    )
    a_communicator->detach(this);
  }
}

std::string comm::Handler::asString() const
throw() {
  string msg("comm::Handler { ");
  msg += Runnable::asString();

  if(a_type == Type::Custom)
    msg += " | Type: Custom";

  msg += functions::asString(" | fd: %d", a_fd);

  if(a_timeout > 0) {
    msg += " | Timeout: ";
    msg += a_timeout.asString();
    msg += " ms";
  }

  return msg += " }";
}

xml::Node* comm::Handler::asXML(xml::Node* parent) const
throw(RuntimeException) {
  xml::Node* result = parent->createChild("comm.Handler");
  asAttribute(result);
  return result;
}

void comm::Handler::asAttribute(xml::Node* node) const
throw(RuntimeException) {
  node->createAttribute("Id", getId());
  node->createAttribute("RequestStop", anna::functions::asString(hasRequestedStop()));

  if(a_type == Type::Custom)
    node->createAttribute("Type", "Custom");

  node->createAttribute("fd", a_fd);

  if(supportTimeout())
    node->createAttribute("Timeout", a_timeout / 1000);
  else
    node->createAttribute("Timeout", "none");
}

