// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/functions.hpp>

// Local
#include <anna/core/tracing/TraceLevelChecker.hpp>
#include <anna/core/tracing/Configuration.hpp>

// STL
#include <map>
#include <string>


using namespace anna::tracing;
using namespace anna;



//------------------------------------------------------------------------------
//------------------------------------ TraceLevelChecker::changeLevelCondition()
//------------------------------------------------------------------------------
bool TraceLevelChecker::changeLevelCondition(const std::string & contextData, anna::Logger::Level & targetLevel) const throw() {
  Configuration & conf = Configuration::instantiate();
  Configuration::trace_trigger_map_t::const_iterator it;
  Configuration::trace_trigger_map_t & traceTriggers = (Configuration::trace_trigger_map_t &)conf.getTraceTriggers();

  if(traceTriggers.size() != 0) {
    if(conf.readRegexpForTraceTriggers()) {
      for(it = traceTriggers.begin(); it != traceTriggers.end(); ++it) {
        //std::string triggerReference = (*it).first;
        anna::RegularExpression & triggerReference = (anna::RegularExpression &)(*it).first;

        // /*contain*/ if (contextData.find(triggerReference) != std::string::npos /*-1*/) {
        //if (anna::functions::isLike(triggerReference.c_str(), contextData)) {
        if(triggerReference.isLike(contextData)) {
          targetLevel = (*it).second;
          return true;
        }
      }
    } else {
      Configuration::trace_trigger_map_t::const_iterator it = traceTriggers.find(contextData);

      if(it != traceTriggers.end()) {  // found (strict match)
        targetLevel = (*it).second;
        return true;
      }
    }
  }

  return (false);
}


//------------------------------------------------------------------------------
//---------------------------------------------------- TraceLevelChecker::load()
//------------------------------------------------------------------------------
bool TraceLevelChecker::load(const char * contextData) throw() {
  a_previousLevel = Logger::getLevel();

  if(contextData) a_contextData = contextData;  // update with valid string provided

  anna::Logger::Level targetLevel;

  if(changeLevelCondition(a_contextData, targetLevel)) {
    Logger::setLevel(targetLevel);
    LOGDEBUG
    (
      std::string msg = "TraceLevelChecker::changeLevelCondition() is true for selective tracing: context data = '";
      msg += a_contextData;
      msg += "', trigger reference set = '";
      msg += (Configuration::instantiate()).asString();
      msg += "'";
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    LOGNOTICE(Logger::notice(anna::functions::asString("Dynamic trace level changed (%s -> %s): context matchs condition for selective tracing", Logger::asString(a_previousLevel), Logger::asString(targetLevel)), ANNA_FILE_LOCATION));
    return (true);
  }

  return (false);
}


//------------------------------------------------------------------------------
//------------------------------------------------- TraceLevelChecker::restore()
//------------------------------------------------------------------------------
bool TraceLevelChecker::restore() throw() {
  anna::Logger::Level currentLevel = Logger::getLevel();

  if(currentLevel != a_previousLevel /* level at constructor begining */) {
    LOGNOTICE(Logger::notice(anna::functions::asString("Dynamic trace level restored (%s -> %s)", Logger::asString(currentLevel), Logger::asString(a_previousLevel)), ANNA_FILE_LOCATION));
    Logger::setLevel(a_previousLevel);
    return (true);
  }

  return (false);
}

